#ifndef MESH_HPP
#define MESH_HPP

#include <SDL/SDL.h>
#include <GL/glu.h>
#include <GL/glext.h>
typedef unsigned short int usint;

struct NFaces{
  usint f1,f2;
};

struct TFaces{
  usint n;
  usint f[4];
};

struct TEdges{
  usint n;
  usint e[4];
};

struct Vertice{
  float x,y,z;
};

struct Edge{
  usint v1,v2;
};

struct Face{
  usint v1,v2,v3,v4;
};

struct Normale{
  float x,y,z;
};

class Mesh{
public:
  usint n_v;
  usint n_e;
  usint n_f;
  int catmull_clark_state;
  Vertice* vertices;
  Edge* edges;
  Face* faces;
  Normale* normales;
  Vertice* face_points; 
  Vertice* edge_points;
  Mesh* sub_mesh;
  
  Mesh(usint n_v,usint n_e,usint n_f);
  void compute_normales();
  void draw();
  Mesh* next_catmull_clark_state();
  Vertice barycentre(usint,usint,usint,usint);
  Vertice barycentre(usint,usint,Vertice,Vertice);
  Vertice barycentre(Vertice,Vertice,Vertice);
  Vertice barycentre(Vertice,Vertice,Vertice,Vertice);
  Vertice face_point(usint f);
  void compute_face_points();
  NFaces get_neighbouring_faces(usint e);
  bool face_contains_vertice(usint f,usint v);
  bool face_contains_edge(usint f,usint e);
  bool edge_contains_vertice(usint e,usint v);
  usint edge_containing_vertices(usint v1,usint v2);
  Vertice edge_point(usint e);
  void compute_edge_points();
  Vertice edge_mid_point(usint e);
  TFaces get_touching_faces(usint v);
  TEdges get_touching_edges(usint v);
  Vertice get_transform_of_orginial_vertice(usint v);
  void create_sub_mesh();
  void compute_edges_of_submesh();
  void compute_faces_of_submesh();
};

#endif
